/* cc-kb-layout-dialog.c
 *
 * Copyright 2019 Bastien Nocera <hadess@hadess.net>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#include <glib/gi18n.h>
#include <adwaita.h>

#include "cc-kb-layout-dialog.h"
#include "cc-keyboard-item.h"

static const InSrcModifier KB_MODIFIER = {
  N_("Switch input source"),
  N_("Switch to next input source"),
  N_("Switch to previous input source"),
  (InSrcOpt[]){
    { NC_("keyboard key", "Alt+Shift"),    "<Alt>Shift_L", "<Shift>Alt_R", "<Alt>Shift_R", "<Shift>Alt_L", TRUE },
    { NC_("keyboard key", "Control+Shift"),   "<Control>Shift_L", "<Shift>Control_R",  "<Control>Shift_R", "<Shift>Control_L", FALSE },
    { NC_("keyboard key", "Control+Alt"),   "<Control>Alt_L", "<Alt>Control_R", "<Control>Alt_R", "<Alt>Control_L", FALSE },
    { NC_("keyboard key", "Alt+Shift_L"),  "<Alt>Shift_L", NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Alt+Shift_R"), "<Alt>Shift_R",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Control+Shift_L"), "<Control>Shift_L",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Control+Shift_R"), "<Control>Shift_R",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Control+Win"),   "<Control>Super_L", "<Super>Control_R",  "<Control>Super_R", "<Super>Control_L", FALSE },
    { NC_("keyboard key", "Alt+Win"),   "<Alt>Super_L", "<Super>Alt_R",  "<Alt>Super_R", "<Super>Alt_L", FALSE },
    { NC_("keyboard key", "Shift+Win"),   "<Shift>Super_L", "<Super>Shift_R",  "<Shift>Super_R", "<Super>Shift_L", FALSE },
    { NC_("keyboard key", "Both Alt together"), "<Alt>Alt_L", "<Alt>Alt_R", NULL, NULL, FALSE },
    { NC_("keyboard key", "Both Control together"), "<Control>Control_L", "<Control>Control_R", NULL, NULL, FALSE },
    { NC_("keyboard key", "Both Shift together"), "<Shift>Shift_L", "<Shift>Shift_R",  NULL, NULL, FALSE },
    { NC_("keyboard key", "Alt+Caps Lock"), "<Alt>Caps_Lock",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Control+Caps Lock"), "<Control>Caps_Lock",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Shift+Caps Lock"),  "<Shift>Caps_Lock",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Win+Space"), "<Super>space",  NULL, NULL, NULL, FALSE },
    { NC_("keyboard key", "Control+Space"), "<Control>space",  NULL, NULL, NULL, FALSE },
    { NULL, NULL, NULL, NULL, NULL, FALSE }
  }
};

struct _CcKbLayoutDialog
{
  AdwWindow   parent_instance;

  AdwPreferencesPage *kb_layout_dialog_page;
  AdwPreferencesGroup *options_group;

  GSettings      *input_source_settings;
  const InSrcModifier *modifier;
  GSList         *radio_group;
  CcKeyboardItem     *item;
  gboolean backward;
};

G_DEFINE_TYPE (CcKbLayoutDialog, cc_kb_layout_dialog, ADW_TYPE_WINDOW)

static GtkCheckButton *
get_radio_button_default (CcKbLayoutDialog *self)
{
  GSList *l;

  for (l = self->radio_group; l != NULL; l = l->next)
    {
      if(g_object_get_data (l->data, "default-opt") != NULL)
        return l->data;
    }
  return NULL;
}

static GtkCheckButton *
get_radio_button_from_in_src_option (CcKbLayoutDialog *self)
{
  gchar *sw_option_1;
  gchar *sw_option_2;
  gchar *sw_option_3;
  gchar *sw_option_4;
  GSList *l;
  g_auto(GStrv) options = NULL;

  if(self->backward) {
    options = g_settings_get_strv (self->input_source_settings, "switch-input-source-backward");
  } else {
    options = g_settings_get_strv (self->input_source_settings, "switch-input-source");
  }

  if(options == NULL)
    return NULL;

  if(options[0] == NULL)
    return NULL;

  for (l = self->radio_group; l != NULL; l = l->next)
  {
    sw_option_1 = g_object_get_data (l->data, "switch-input-source-1");
    sw_option_2 = g_object_get_data (l->data, "switch-input-source-2");
    sw_option_3 = g_object_get_data (l->data, "switch-input-source-3");
    sw_option_4 = g_object_get_data (l->data, "switch-input-source-4");
    if (g_strcmp0 (sw_option_1, options[0]) == 0) {
      if(options[1] != NULL && sw_option_2 != NULL) {
        if (g_strcmp0 (sw_option_2, options[1]) == 0) {
          if(options[2] != NULL && sw_option_3 != NULL) {
            if (g_strcmp0 (sw_option_3, options[2]) == 0) {
              if(options[3] != NULL && sw_option_4 != NULL) {
                if (g_strcmp0 (sw_option_4, options[3]) == 0)
                  return l->data;
              }
            }
          }
        }
      }
    }
  }

  for (l = self->radio_group; l != NULL; l = l->next)
  {
    sw_option_1 = g_object_get_data (l->data, "switch-input-source-1");
    sw_option_2 = g_object_get_data (l->data, "switch-input-source-2");
    sw_option_3 = g_object_get_data (l->data, "switch-input-source-3");
    if (g_strcmp0 (sw_option_1, options[0]) == 0) {
      if(options[1] != NULL && sw_option_2 != NULL) {
        if (g_strcmp0 (sw_option_2, options[1]) == 0) {
          if(options[2] != NULL && sw_option_3 != NULL) {
            if (g_strcmp0 (sw_option_3, options[2]) == 0)
              return l->data;
            }
          }
       }
    }
  }

  for (l = self->radio_group; l != NULL; l = l->next)
  {
    sw_option_1 = g_object_get_data (l->data, "switch-input-source-1");
    sw_option_2 = g_object_get_data (l->data, "switch-input-source-2");
    if (g_strcmp0 (sw_option_1, options[0]) == 0) {
      if(options[1] != NULL && sw_option_2 != NULL) {
        if (g_strcmp0 (sw_option_2, options[1]) == 0)
          return l->data;
      }
    }
  }

  for (l = self->radio_group; l != NULL; l = l->next)
  {
    sw_option_1 = g_object_get_data (l->data, "switch-input-source-1");
    if (g_strcmp0 (sw_option_1, options[0]) == 0)
       return l->data;
  }

  return NULL;
}

static void
update_active_radio (CcKbLayoutDialog *self)
{
  g_auto(GStrv) options = NULL;
  GtkCheckButton *rightalt_radio;
  GtkCheckButton *radio;

  if(self->item != NULL) {
    if (g_strcmp0 (cc_keyboard_item_get_key(self->item), "switch-input-source-backward") == 0) {
      adw_preferences_page_set_description (self->kb_layout_dialog_page, gettext (self->modifier->description_back));
      self->backward = TRUE;
    } else {
      adw_preferences_page_set_description (self->kb_layout_dialog_page, gettext (self->modifier->description));
      self->backward = FALSE;
    }
  }

   radio = get_radio_button_from_in_src_option (self);

  if (radio) {
    gtk_check_button_set_active (GTK_CHECK_BUTTON (radio), TRUE);

    return;
  }

  rightalt_radio = get_radio_button_default(self);
  if(rightalt_radio != NULL) {
    gtk_check_button_set_active (GTK_CHECK_BUTTON (rightalt_radio), TRUE);

  }
}

static void
set_in_src_option (CcKbLayoutDialog *self,
               gchar               *switch_input_source_opt_1,
               gchar               *switch_input_source_opt_2,
               gchar               *switch_input_source_opt_3,
               gchar               *switch_input_source_opt_4)
{
  g_autoptr(GPtrArray) array_b = NULL;

  array_b = g_ptr_array_new ();
  if(switch_input_source_opt_1 != NULL) {
    g_ptr_array_add (array_b, switch_input_source_opt_1);

      if(switch_input_source_opt_2 != NULL)
        g_ptr_array_add (array_b, switch_input_source_opt_2);

      if(switch_input_source_opt_3 != NULL)
        g_ptr_array_add (array_b, switch_input_source_opt_3);

      if(switch_input_source_opt_4 != NULL)
        g_ptr_array_add (array_b, switch_input_source_opt_4);

      g_ptr_array_add (array_b, NULL);
      if(self->backward) {
        g_settings_set_strv (self->input_source_settings,
                      "switch-input-source-backward",
                      (const gchar * const *) array_b->pdata);
      } else {
                g_settings_set_strv (self->input_source_settings,
                      "switch-input-source",
                      (const gchar * const *) array_b->pdata);
      }
  }
}

static void
on_active_radio_changed_cb (CcKbLayoutDialog *self,
                            GtkCheckButton    *radio)
{
  gchar *switch_input_source_opt_1;
  gchar *switch_input_source_opt_2;
  gchar *switch_input_source_opt_3;
  gchar *switch_input_source_opt_4;

  if (!gtk_check_button_get_active (GTK_CHECK_BUTTON (radio)))
    return;

  switch_input_source_opt_1 = (gchar *)g_object_get_data (G_OBJECT (radio), "switch-input-source-1");
  switch_input_source_opt_2 = (gchar *)g_object_get_data (G_OBJECT (radio), "switch-input-source-2");
  switch_input_source_opt_3 = (gchar *)g_object_get_data (G_OBJECT (radio), "switch-input-source-3");
  switch_input_source_opt_4 = (gchar *)g_object_get_data (G_OBJECT (radio), "switch-input-source-4");
  set_in_src_option (self, switch_input_source_opt_1, switch_input_source_opt_2, switch_input_source_opt_3, switch_input_source_opt_4);
}

static void
on_in_src_options_changed_cb (CcKbLayoutDialog *self)
{
  if (self->modifier == NULL)
    update_active_radio (self);
}

static void
cc_kb_layout_dialog_finalize (GObject *object)
{
  CcKbLayoutDialog *self = (CcKbLayoutDialog *)object;

  g_clear_object (&self->input_source_settings);

  G_OBJECT_CLASS (cc_kb_layout_dialog_parent_class)->finalize (object);
}

static void
cc_kb_layout_dialog_class_init (CcKbLayoutDialogClass *klass)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
  GObjectClass *object_class = G_OBJECT_CLASS (klass);

  object_class->finalize = cc_kb_layout_dialog_finalize;

  gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/control-center/keyboard/cc-kb-layout-dialog.ui");

  gtk_widget_class_bind_template_child (widget_class, CcKbLayoutDialog, kb_layout_dialog_page);
  gtk_widget_class_bind_template_child (widget_class, CcKbLayoutDialog, options_group);
}

static void
add_radio_buttons (CcKbLayoutDialog *self)
{
  g_autoptr (GSList) group = NULL;
  GtkWidget *row, *radio_button, *last_button = NULL;
  InSrcOpt *options = self->modifier->options;
  int i;

  for (i = 0; options[i].label && options[i].switch_input_source_1; i++)
    {
        row = g_object_new (ADW_TYPE_ACTION_ROW, "selectable", FALSE, NULL);
        adw_preferences_group_add (self->options_group, row);
        radio_button = g_object_new (GTK_TYPE_CHECK_BUTTON, "valign", GTK_ALIGN_CENTER, "group", last_button, NULL);

       g_object_set_data (G_OBJECT (radio_button), "switch-input-source-1", options[i].switch_input_source_1);
       g_object_set_data (G_OBJECT (radio_button), "switch-input-source-2", options[i].switch_input_source_2);
       g_object_set_data (G_OBJECT (radio_button), "switch-input-source-3", options[i].switch_input_source_3);
       g_object_set_data (G_OBJECT (radio_button), "switch-input-source-4", options[i].switch_input_source_4);

      if(options[i].default_opt) {
        g_object_set_data (G_OBJECT (radio_button), "default-opt", "default");
      } else {
        g_object_set_data (G_OBJECT (radio_button), "default-opt", NULL);
      }
       g_signal_connect_object (radio_button, "toggled", (GCallback)on_active_radio_changed_cb, self, G_CONNECT_SWAPPED);
       adw_action_row_add_prefix (ADW_ACTION_ROW (row), radio_button);
       adw_preferences_row_set_title (ADW_PREFERENCES_ROW (row), options[i].label);
       adw_action_row_set_activatable_widget (ADW_ACTION_ROW (row), radio_button);

       last_button = radio_button;
      group = g_slist_prepend (group, radio_button);
    }

  self->radio_group = NULL;
  if (last_button != NULL)
    self->radio_group = g_steal_pointer (&group);
}

static void
cc_kb_layout_dialog_init (CcKbLayoutDialog *self)
{
  gtk_widget_init_template (GTK_WIDGET (self));

  self->modifier = &KB_MODIFIER;

  self->input_source_settings = g_settings_new ("org.gnome.desktop.wm.keybindings");

  g_signal_connect_object (self->input_source_settings,
                           "changed::switch-input-source-backward",
                           G_CALLBACK (on_in_src_options_changed_cb),
                           self, G_CONNECT_SWAPPED);
}

CcKbLayoutDialog *
cc_kb_layout_dialog_new (GtkWindow  *parent_window)
{
  CcKbLayoutDialog *self;
  InSrcModifier *modifier;

  modifier = &KB_MODIFIER;
  self = g_object_new (CC_TYPE_KB_LAYOUT_DIALOG, "transient-for", parent_window, NULL);

  add_radio_buttons (self);
  update_active_radio (self);


  return self;
}

void
cc_kb_layout_dialog_set_item (CcKbLayoutDialog *self,
                                      CcKeyboardItem           *item)
{
  self->item = item;
  update_active_radio (self);
}
